import annotate from "./annotate";
function register(kernel) {
    return function (identifier, dependencies, constraint) {
        if (dependencies === void 0) { dependencies = []; }
        return function (constructor) {
            annotate(constructor, dependencies);
            var binding = kernel.bind(identifier).to(constructor);
            if (constraint !== undefined) {
                constraint(binding);
            }
        };
    };
}
function registerSelf(kernel) {
    return function (dependencies, constraint) {
        if (dependencies === void 0) { dependencies = []; }
        return function (constructor) {
            annotate(constructor, dependencies);
            var binding = kernel.bind(constructor).toSelf();
            if (constraint !== undefined) {
                constraint(binding);
            }
        };
    };
}
function registerConstructor(kernel) {
    return function (identifier, constraint) {
        return function (constructor) {
            var binding = kernel.bind(identifier).toConstructor(constructor);
            if (constraint !== undefined) {
                constraint(binding);
            }
        };
    };
}
function registerConstantValue(kernel) {
    return function (identifier, value, constraint) {
        var binding = kernel.bind(identifier).toConstantValue(value);
        if (constraint !== undefined) {
            constraint(binding);
        }
    };
}
function registerDynamicValue(kernel) {
    return function (identifier, func, constraint) {
        var binding = kernel.bind(identifier).toDynamicValue(func);
        if (constraint !== undefined) {
            constraint(binding);
        }
    };
}
function registerFunction(kernel) {
    return function (identifier, func, constraint) {
        var binding = kernel.bind(identifier).toFunction(func);
        if (constraint !== undefined) {
            constraint(binding);
        }
    };
}
function registerAutoFactory(kernel) {
    return function (factoryIdentifier, serviceIdentifier, constraint) {
        var binding = kernel.bind(factoryIdentifier).toAutoFactory(serviceIdentifier);
        if (constraint !== undefined) {
            constraint(binding);
        }
    };
}
function registerFactory(kernel) {
    return function (identifier, factory, constraint) {
        var binding = kernel.bind(identifier).toFactory(factory);
        if (constraint !== undefined) {
            constraint(binding);
        }
    };
}
function registerProvider(kernel) {
    return function (identifier, provider, constraint) {
        var binding = kernel.bind(identifier).toProvider(provider);
        if (constraint !== undefined) {
            constraint(binding);
        }
    };
}
export { register, registerSelf, registerConstantValue, registerDynamicValue, registerConstructor, registerFunction, registerAutoFactory, registerFactory, registerProvider };
