"use strict";
var annotate_1 = require("./annotate");
function register(kernel) {
    return function (identifier, dependencies, constraint) {
        if (dependencies === void 0) { dependencies = []; }
        return function (constructor) {
            annotate_1.default(constructor, dependencies);
            var binding = kernel.bind(identifier).to(constructor);
            if (constraint !== undefined) {
                constraint(binding);
            }
        };
    };
}
exports.register = register;
function registerSelf(kernel) {
    return function (dependencies, constraint) {
        if (dependencies === void 0) { dependencies = []; }
        return function (constructor) {
            annotate_1.default(constructor, dependencies);
            var binding = kernel.bind(constructor).toSelf();
            if (constraint !== undefined) {
                constraint(binding);
            }
        };
    };
}
exports.registerSelf = registerSelf;
function registerConstructor(kernel) {
    return function (identifier, constraint) {
        return function (constructor) {
            var binding = kernel.bind(identifier).toConstructor(constructor);
            if (constraint !== undefined) {
                constraint(binding);
            }
        };
    };
}
exports.registerConstructor = registerConstructor;
function registerConstantValue(kernel) {
    return function (identifier, value, constraint) {
        var binding = kernel.bind(identifier).toConstantValue(value);
        if (constraint !== undefined) {
            constraint(binding);
        }
    };
}
exports.registerConstantValue = registerConstantValue;
function registerDynamicValue(kernel) {
    return function (identifier, func, constraint) {
        var binding = kernel.bind(identifier).toDynamicValue(func);
        if (constraint !== undefined) {
            constraint(binding);
        }
    };
}
exports.registerDynamicValue = registerDynamicValue;
function registerFunction(kernel) {
    return function (identifier, func, constraint) {
        var binding = kernel.bind(identifier).toFunction(func);
        if (constraint !== undefined) {
            constraint(binding);
        }
    };
}
exports.registerFunction = registerFunction;
function registerAutoFactory(kernel) {
    return function (factoryIdentifier, serviceIdentifier, constraint) {
        var binding = kernel.bind(factoryIdentifier).toAutoFactory(serviceIdentifier);
        if (constraint !== undefined) {
            constraint(binding);
        }
    };
}
exports.registerAutoFactory = registerAutoFactory;
function registerFactory(kernel) {
    return function (identifier, factory, constraint) {
        var binding = kernel.bind(identifier).toFactory(factory);
        if (constraint !== undefined) {
            constraint(binding);
        }
    };
}
exports.registerFactory = registerFactory;
function registerProvider(kernel) {
    return function (identifier, provider, constraint) {
        var binding = kernel.bind(identifier).toProvider(provider);
        if (constraint !== undefined) {
            constraint(binding);
        }
    };
}
exports.registerProvider = registerProvider;
